/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.jedule.parser.JeduleParserException;
import net.sf.jedule.schedule.Jedule;
import net.sf.jedule.schedule.JeduleNode;
import net.sf.jedule.settings.JeduleImageInformation;
import net.sf.jedule.settings.JeduleSaveInformation;
import net.sf.jedule.settings.global.JeduleSettings;
import net.sf.jedule.ui.IJSheetGraphicsContext;
import net.sf.jedule.ui.JSheetGraphicsContextFactory;
import net.sf.jedule.ui.UnsupportedContextException;
import net.sf.jedule.ui.event.IJSheetContextListener;
import net.sf.jedule.ui.event.JSheetClusterSelectionEvent;
import net.sf.jedule.ui.event.JSheetClusterSelectionEvent2;
import net.sf.jedule.ui.event.JSheetContextEvent;
import net.sf.jedule.ui.event.JSheetLoadContextEvent;
import net.sf.jedule.ui.event.JSheetNodeInfoContextEvent;

/**
 * @author hunold
 *
 */
public class JeduleController implements IJSheetContextListener {

    private IJSheetGraphicsContext graphicsContext;

    private final static Logger LOGGER = Logger
            .getLogger(JeduleController.class.getName());

    public JeduleController(final JeduleOptions jeduleOptions)
            throws JeduleParserException, UnsupportedContextException {

        final Jedule jedule = loadJedule(jeduleOptions.getJeduleFileName(),
                jeduleOptions.getJeduleFileType());

        JeduleSettings.getInstance().init(jeduleOptions);
        initGraphicsContext(jeduleOptions);

        final List<Integer> clusterIdList = new ArrayList<Integer>();
        final int nb_of_clusters = jedule.getJeduleGrid().getNumberOfClusters();
        for (int i = 0; i < nb_of_clusters; i++) {
            clusterIdList.add(i);
        }

        graphicsContext.addIJSheetContextListener(this);
        graphicsContext.setJedule(jedule);

        if (jeduleOptions.getOutputFile() == null) {
            graphicsContext.show(jeduleOptions.getWidth(), jeduleOptions
                    .getHeight());
        } else {
            try {

                graphicsContext.normalize();
                final JeduleSaveInformation info = new JeduleSaveInformation(jeduleOptions.getOutputFile(), jeduleOptions.getOutputType());
                final JeduleImageInformation imageSpec = new JeduleImageInformation();
                imageSpec.setImageWidth(jeduleOptions.getWidth());
                imageSpec.setImageHeight(jeduleOptions.getHeight());
                info.setJeduleImageInformation(imageSpec);

                graphicsContext.save(info);
            } catch (final IOException e) {
                LOGGER.severe("Cannot write output file "
                        + jeduleOptions.getOutputFile());
                LOGGER.severe(e.getMessage());
            } finally {
                graphicsContext.close();
            }
        }
    }

    /**
     * @param jeduleOptions
     * @throws UnsupportedContextException
     */
    private void initGraphicsContext(final JeduleOptions jeduleOptions)
            throws UnsupportedContextException {

        this.graphicsContext = JSheetGraphicsContextFactory
                .getGraphicsContext(jeduleOptions.getGraphicsContext());

    }

    /**
     * @param jeduleFileName
     * @param jeduleFileType
     * @throws JeduleParserException
     */
    private Jedule loadJedule(final String jeduleFileName,
            final String jeduleFileType) throws JeduleParserException {
        return new Jedule(new JeduleXmlModel(jeduleFileName, jeduleFileType),
                jeduleFileName);
    }

    /*
     * (non-Javadoc)
     *
     * @see net.sf.jedule.ui.IJSheetContextListener#performContextEvent(net.sf.jedule.ui.IJSheetContextEvent)
     */
    public void performContextEvent(final JSheetContextEvent e) {

        final String cmd = e.getCommand();

        if ( cmd.equals(JSheetContextEvent.CMD_JEDULE_QUIT)) {

            graphicsContext.close();

        } else if (cmd.equals(JSheetContextEvent.CMD_JEDULE_RELOAD)) {

            LOGGER.info("reload graphic");

            final Jedule jed = graphicsContext.getJedule();
            if (jed != null) {
                try {
                    jed.reload();
                    graphicsContext.update();
                } catch (final JeduleParserException e1) {
                    // e1.printStackTrace();
                    graphicsContext.getJSheetErrorReporter()
                            .reportException(e1);
                }
            }

        } else if (cmd.equals(JSheetContextEvent.CMD_JEDULE_CMAP_CHANGED)) {

            JeduleSettings.getInstance().getJeduleColorConfig()
                    .setActiveJeduleColorMap(e.getValue());

            graphicsContext.update();

        } else if (e instanceof JSheetLoadContextEvent) {

            LOGGER.info("load event");

            final JSheetLoadContextEvent lE = (JSheetLoadContextEvent) e;

            final String fileName = graphicsContext.getFileName();
            if (fileName != null) {

                try {
                    final Jedule jedule = loadJedule(fileName, lE
                            .getParserName());
                    graphicsContext.setJedule(jedule);
                    graphicsContext.update();
                } catch (final JeduleParserException e1) {
                    // e1.printStackTrace();
                    graphicsContext.getJSheetErrorReporter()
                            .reportException(e1);
                }

            }

        } else if (e instanceof JSheetNodeInfoContextEvent) {

            final JSheetNodeInfoContextEvent infoEvent = (JSheetNodeInfoContextEvent) e;
            graphicsContext.showNodeInformation(infoEvent.getNodeIds());

        } else if (e instanceof JSheetClusterSelectionEvent) {

            final JSheetClusterSelectionEvent jCSE = (JSheetClusterSelectionEvent) e;
            graphicsContext
                    .enableCluster(jCSE.getClusterId(), jCSE.isEnabled());

        } else if (e instanceof JSheetClusterSelectionEvent2) {

            final JSheetClusterSelectionEvent2 jCSE = (JSheetClusterSelectionEvent2) e;
            graphicsContext.showClusters(jCSE.getIdList());

        } else if (cmd.equals(JSheetContextEvent.MARK_NODE)) {

            markNodes((String) e.getValue());

        } else if (cmd.equals(JSheetContextEvent.MAKE_SNAPSHOT)) {

            try {
                graphicsContext.makeSnapshot();
            } catch (IOException ie) {
                LOGGER.severe("error while creating snapshot");
                ie.printStackTrace();
            }


        } else if (cmd.equals(JSheetContextEvent.SAVE_IMAGE)) {

            final JeduleSaveInformation info = graphicsContext.getJeduleSaveInformation();
            if (info != null) {

                LOGGER.fine("saving image to file : " + info.getFileName());

                try {
                    graphicsContext.save(info);
                } catch (IOException ioe) {
                    LOGGER.severe("cannot save file : " + ioe.getMessage());
                    ioe.printStackTrace();

                    graphicsContext.showErrorDialog(ioe.getMessage());
                }

            }

        } else {
            LOGGER.warning("unknown command " + e.getCommand());
        }

    }

    /**
     * @param value
     */
    private void markNodes(final String regexString) {

        final Jedule jedule = graphicsContext.getJedule();
        final Pattern p = Pattern.compile(regexString);

        final List<String> nodeNamesToMark = new ArrayList<String>();
        for (final JeduleNode node : jedule.getJeduleNodeList()) {
            final Matcher matcher = p.matcher(node.getId());
            if (matcher.matches()) {
                nodeNamesToMark.add(node.getId());
            }
        }

        graphicsContext.markNodes(nodeNamesToMark);

    }

}
