/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.graphics.composed;

import java.awt.Color;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Logger;

import net.sf.jedule.graphics.GraphicsConstants;
import net.sf.jedule.graphics.JSheetObject;
import net.sf.jedule.graphics.JSheetRectangle;
import net.sf.jedule.graphics.GraphicsConstants.ZOOM_DIRECTION;
import net.sf.jedule.schedule.Jedule;
import net.sf.jedule.schedule.JeduleNode;
import net.sf.jedule.schedule.JeduleTask;
import net.sf.jedule.schedule.JeduleToolkit;
import net.sf.jedule.settings.global.JeduleColorMap;
import net.sf.jedule.settings.global.JeduleSettings;
import net.sf.jedule.settings.global.parser.ColorMapParser;

/**
 * @author sascha
 *
 */
public class JSheetRuler extends JSheetObject {

  private static final int LEFT_RULER_OFFSET = 50;
    private static final int RIGHT_RULER_OFFSET = 20;
    private static final int BOTTOM_RULER_OFFSET = 50;
    private static final int TOP_RULER_OFFSET = 20;

    private static final int TIC_LENGTH = 2;
    private static final int TICS_NUM_Y = 10;
    private static final int ARROW_LENGTH = 10;

    private Jedule jedule;
    private int clusterId;
    private Rectangle zoomBounds;

    private final double globalYMin;
    private final double globalYMax;
    private final double globalXMin;
    private final double globalXMax;

    private double yMin; // smallest y value (0 is at bottom)
    private double yMax;
    private double xMin;
    private double xMax;

    // maps a task to its labels
    private final HashMap<JeduleTask, JSheetTaskRectangle> task2RectMap = new HashMap<JeduleTask, JSheetTaskRectangle>();
    private List<JeduleTask> taskLayoutList;

    private final static Logger LOGGER = Logger.getLogger(JSheetRuler.class
            .getName());

    public JSheetRuler(Jedule jedule, int clusterId, int width, int height) {
        super(width, height);
        this.jedule = jedule;
        this.clusterId = clusterId;
        this.globalXMin = jedule.getJeduleGrid().getClusterById(clusterId)
                .getFirstHost();
        this.globalXMax = jedule.getJeduleGrid().getClusterById(clusterId)
                .getFirstHost()
                + jedule.getJeduleGrid().getClusterById(clusterId)
                        .getNumberOfHosts();
        this.globalYMin = 0.0;
        this.globalYMax = jedule.getFinishTime(clusterId);
        initView();
    }

    /**
     * initialized view to default view call after zoom to get the standard view
     * back
     */
    public void initView() {
        initGrid();
        setIntervalX(this.globalXMin, this.globalXMax);
        setIntervalY(this.globalYMin, this.globalYMax);
    }

    private void initGrid() {
        yMin = globalYMin;
        yMax = globalYMax;
        xMin = globalXMin;
        xMax = globalXMax;
    }

    public int getLeftOffset() {
        return LEFT_RULER_OFFSET;
    }

    public int getRightOffset() {
        return RIGHT_RULER_OFFSET;
    }

    public int getBottomOffset() {
        return BOTTOM_RULER_OFFSET;
    }

    public int getTopOffset() {
        return TOP_RULER_OFFSET;
    }

    public int getNumberOfTicsX() {
        int firstProc = (int) Math.floor(getMinX());
        int lastProc = (int) Math.ceil(getMaxX());
        return (lastProc - firstProc);
    }

    public String getXLabelAt(int tic) {
        int firstProc = (int) Math.ceil(getMinX());

        // LOGGER.fine("first xtic value : " + getFirstXTicValue());
        if (getMinX() > globalXMin) {
            // the first tic is not the end of the first processor
            // but the end of its predecessor
            firstProc--;
            // LOGGER.info("first proc--");
        }
        return Integer.toString(firstProc + tic);
    }

    public int getTicLength() {
        return TIC_LENGTH;
    }

    public double getNumberOfTicsY() {
        return TICS_NUM_Y;
    }

    public int getArrowLength() {
        return ARROW_LENGTH;
    }

    public int getLengthAxisX() {
        return getWidth() - getLeftOffset() - getRightOffset();
    }

    public int getLengthAxisY() {
        return getHeight() - getTopOffset() - getBottomOffset();
    }

    private void insertNodes() {
        final List<JeduleTask> taskList = new ArrayList<JeduleTask>();
        for (JeduleNode node : jedule.getJeduleNodeList()) {
            if (node.getClusterId() == clusterId) {
                List<JeduleTask> nodeTasks = node.getJeduleTaskList();
                taskList.addAll(nodeTasks);
            }
        }

        final List<List<JeduleTask>> listOfTaskL = JeduleToolkit
                .getOverlappingTasks(taskList);
        for (final List<JeduleTask> list : listOfTaskL) {
            if (list.size() == 0) {
                LOGGER.warning("BUG task list must not be empty");
            } else {
                if (list.size() <= 0) {
                    // bullshit
                    LOGGER.severe("BUG!");
                } else if (list.size() == 1) {
                    JeduleTask firstTask = list.get(0);
                    insertTask(firstTask);
                } else {
//                    for(JeduleTask t : list) {
//                        System.out.println("overlapped : " + t);
//                    }
                    insertOverlappedTasks(list);
                }
            }
        }
    }

    private void insertOverlappedTasks(final List<JeduleTask> taskList) {
        final List<List<JeduleTask>> listMiniTaskList = JeduleToolkit
                .partitionTasks(taskList);

        for (final List<JeduleTask> l : listMiniTaskList) {

            if (l.size() == 0) {
                continue;
            } else if (l.size() == 1) {
                //System.out.println("draw: " + l);
                insertTask(l.get(0));
            } else {
                final JeduleTask firstTask = l.get(0);
                final List<String> idList = new ArrayList<String>();
                for (final JeduleTask t : l) {
                    idList.add(t.getNodeId());
                }
                Collections.sort(idList);
                final String[] nodeIds = idList.toArray(new String[idList
                        .size()]);

                final Rectangle taskRect = placeTask(firstTask);

                // is it composed of different types
                final List<String> taskTypeList = new ArrayList<String>();
                for (final JeduleTask t : l) {
                    taskTypeList.add(t.getNodeType());
                }

                JSheetTaskRectangle sheetRectangle = new JSheetTaskRectangle(
                        taskRect.width, taskRect.height, nodeIds, nodeIds);

                colorRectangle(sheetRectangle, taskTypeList);

                saveTask(firstTask, sheetRectangle);
            }
        }
    }

    private void saveTask(final JeduleTask task,
            final JSheetTaskRectangle rectangle) {
        taskLayoutList.add(task);
        task2RectMap.put(task, rectangle);
    }

    private void insertTask(final JeduleTask task) {
        Rectangle taskRect = placeTask(task);

        JSheetTaskRectangle rectangle = new JSheetTaskRectangle(taskRect.width,
                taskRect.height, new String[] { task.getNodeId() },
                new String[] { task.getNodeId() });
        
        final List<String> taskTypeList = new ArrayList<String>();
        taskTypeList.add(task.getNodeType());
        
        colorRectangle(rectangle, taskTypeList);
        saveTask(task, rectangle);
    }

    private void colorRectangle(final JSheetRectangle rect,
            final List<String> taskTypeList) {
        final JeduleColorMap colorMap = JeduleSettings.getInstance()
                .getJeduleColorConfig().getActiveJeduleColorMap();        
        
        String taskType = null;
        if( taskTypeList.size() == 1 ) {
            taskType = taskTypeList.get(0);
        } else if( taskTypeList.size() < 1 ) { 
            LOGGER.warning("BUG! There must be at least 1 task type");
        } else {
            // composite
            taskType = ColorMapParser.getHashKeyForComposite(taskTypeList);
        }
        
        final Color fgCol = colorMap.getColor(taskType, "fg");
        final Color bgCol = colorMap.getColor(taskType, "bg");
        
        rect.setForeground(fgCol);
        rect.setBackground(bgCol);
    }

    private Rectangle placeTask(JeduleTask task) {
        int x1 = getScreenPositionX(task.getStartHost());
        int x2 = getScreenPositionX(task.getEndHost() + 1);

        int y1 = getScreenPositionY(task.getStartTime());
        int y2 = getScreenPositionY(task.getEndTime());

        // cut left
        if (x1 < getLeftOffset() && x2 > getLeftOffset()) {
            x1 = getLeftOffset();
        }

        // cut right
        if (x1 < getWidth() - getRightOffset()
                && x2 > getWidth() - getRightOffset()) {
            x2 = getWidth() - getRightOffset();
        }

        // bottom
        if (y1 > getHeight() - getBottomOffset()
                && y2 < getHeight() - getBottomOffset()) {
            y1 = getHeight() - getBottomOffset();
        }

        // top
        if (y2 < getTopOffset() + getArrowLength()
                && y1 > getTopOffset() + getArrowLength()) {
            y2 = getTopOffset() + getArrowLength();
        }

        Rectangle r = new Rectangle();
        // check if rectangle bounds are inside grid bounds
        // if not make rectangle invisible
        if (x1 >= getLeftOffset() && x2 <= getWidth() - getRightOffset()
                && y1 <= getHeight() - getBottomOffset()
                && y2 >= getTopOffset() + getArrowLength()) {

            r.width = Math.abs(x2 - x1);
            r.height = Math.abs(y2 - y1);
            r.x = x1;
            r.y = y2;
        } else {
            r.x = -1;
            r.y = -1;
            r.width = 0;
            r.height = 0;
        }

        LOGGER.fine("task " + task.getNodeId() + " y1=" + y1 + " y2=" + y2
                + " w=" + r.width + " h=" + r.height);

        return r;
    }

    public int getScreenPositionX(double value) {
        int x = -1;
        final double normValue = value - getMinX();
        x = (int) ((normValue * (double) getLengthAxisX()) / (getMaxX() - getMinX()));
        x = x + getLeftOffset();
        return x;
    }

    public int getScreenPositionY(double value) {
        int y = -1;
        final double normalizedTime = value - getMinY();
        y = (int) ((normalizedTime * (double) getLengthAxisY()) / (getMaxY() - getMinY()));
        y = getHeight() - getBottomOffset() - y;
        return y;
    }

    public String getLastTicLabelY() {
        DecimalFormat formatter = new DecimalFormat("##0.0");
        formatter.setDecimalSeparatorAlwaysShown(true);
        return formatter.format(yMax);
    }

    public String getFirstTicLabelY() {
        DecimalFormat formatter = new DecimalFormat("##0.0");
        formatter.setDecimalSeparatorAlwaysShown(true);
        return formatter.format(yMin);
    }

    /**
     * sets bounds of y axis without considering the global minimum and maximum
     * which is defined by the jedule (e.g. for normalizing the views)
     *
     */
    public void setGlobalBoundsY(double yMin, double yMax) {
        this.yMin = yMin;
        this.yMax = yMax;
    }

    public void setBoundsY(double yMin, double yMax) {
        this.yMin = Math.max(globalYMin, yMin);
        this.yMax = Math.min(globalYMax, yMax);
    }

    public void setBoundsX(double xMin, double xMax) {
        this.xMin = Math.max(globalXMin, xMin);
        this.xMax = Math.min(globalXMax, xMax);
    }

    public double getMinY() {
        return yMin;
    }

    public double getMaxY() {
        return yMax;
    }

    public double getMinX() {
        return xMin;
    }

    public double getMaxX() {
        return xMax;
    }

    public double getFirstXTicValue() {
        return Math.floor(xMin);
    }

    public double getLastXTicValue() {
        return Math.ceil(xMax);
    }

    /*
     * (non-Javadoc)
     *
     * @see net.sf.jedule.ui.IJSheetGraphicsContextRenderable#doLayout(int, int)
     */
    public void doLayout() {
        if (taskLayoutList == null) {
            taskLayoutList = new ArrayList<JeduleTask>();
            insertNodes();
        }
        layoutNodes();
    }

    private void layoutNodes() {
        for (JeduleTask t : taskLayoutList) {
            final JSheetTaskRectangle taskRect = task2RectMap.get(t);
            Rectangle rect = placeTask(t);
            taskRect.setVisable( isVisable(rect) );
            taskRect.setSize(rect.width, rect.height);
            addChildObject(taskRect, rect.x, rect.y);
            LOGGER.fine("task " + t.getNodeId() + " rect=" + rect);
        }
    }

    /**
     * @param taskRect
     * @return
     */
    private boolean isVisable(Rectangle rect) {
        boolean visable = true;

        if( rect.getHeight() < 1 ) {
            visable= false;
        } else if (rect.y > getHeight() - getBottomOffset()) {
            visable = false;
        } else if (rect.x > getWidth() - getRightOffset()) {
            visable = false;
        }

        return visable;
    }

    /**
     * @param i
     * @return
     */
    public double getValueForScreenPositionY(int ypos) {
        ypos = getLengthAxisY() - (ypos - getTopOffset());
        if (ypos < 0) {
            ypos = 0;
        } else if (ypos > getLengthAxisY()) {
            ypos = getLengthAxisY();
        }
        double yval = (yMax - yMin) * (double) ypos / (double) getLengthAxisY();
        yval += yMin;
        return yval;
    }

    /**
     * @param x
     * @return
     */
    public double getValueForScreenPositionX(int xpos) {
        xpos -= getLeftOffset();
        if (xpos < 0) {
            xpos = 0;
        } else if (xpos > getLengthAxisX()) {
            xpos = getLengthAxisX();
        }
        double xval = (xMax - xMin) * (double) xpos / (double) getLengthAxisX();
        xval += xMin;
        return xval;
    }

    /**
     * @param minTime
     * @param maxTime
     */
    private void setIntervalY(double minTime, double maxTime) {
        this.yMin = minTime;
        this.yMax = maxTime;
    }

    /**
     * @param minProc
     * @param maxProc
     */
    private void setIntervalX(double minProc, double maxProc) {
        this.xMin = minProc;
        this.xMax = maxProc;
    }

    /**
     * @param rectangle
     */
    public void zoom(Rectangle rectangle) {
        this.zoomBounds = rectangle;

        if (zoomBounds != null) {
            setIntervals(zoomBounds);
        }
    }

    private void setIntervals(Rectangle bounds) {

        double minProc = getValueForScreenPositionX(bounds.x);
        double maxProc = getValueForScreenPositionX(bounds.x + bounds.width);
        double minTime = getValueForScreenPositionY(bounds.y + bounds.height);
        double maxTime = getValueForScreenPositionY(bounds.y);

        LOGGER.fine("xpos : " + (bounds.x + bounds.width) + " value "
                + getValueForScreenPositionX(bounds.x + bounds.width));

        setIntervalX(minProc, maxProc);
        setIntervalY(minTime, maxTime);

        LOGGER.fine("minTime: " + minTime + ", maxTime: " + maxTime
                + ", minProc: " + minProc + ", max Proc: " + maxProc);
    }

    protected Rectangle getRulerBounds() {
        return new Rectangle(getLeftOffset(), getTopOffset(), getLengthAxisX(),
                getLengthAxisY());
    }

    /**
     * @param scaling
     * @param centerPoint
     * @param direction
     */
    public void zoom(float scaling, Point centerPoint,
            GraphicsConstants.ZOOM_DIRECTION direction) {

        double newLengthX = xMax - xMin;
        double newLengthY = yMax - yMin;

        if (direction == ZOOM_DIRECTION.X_AXIS
                || direction == ZOOM_DIRECTION.BOTH_AXES) {
            newLengthX *= scaling;
        }

        if (direction == ZOOM_DIRECTION.Y_AXIS
                || direction == ZOOM_DIRECTION.BOTH_AXES) {
            newLengthY *= scaling;
        }

        zoomInto(centerPoint, newLengthX, newLengthY);
    }

    private void zoomInto(final Point centerPoint, final double newLengthX,
            final double newLengthY) {

        if (!getRulerBounds().contains(centerPoint)) {
            return;
        }

        final double oldLengthX = getMaxX() - getMinX();
        final double oldLengthY = getMaxY() - getMinY();

        final double cValueX = getValueForScreenPositionX(centerPoint.x
                - getLeftOffset());
        final double cValueY = getValueForScreenPositionY(centerPoint.y
                - getTopOffset());

        final double bottomFraction = (cValueY - getMinY())
                / (getMaxY() - getMinY());
        final double topFraction = 1.0 - bottomFraction;

        double newMinY = Math.max(this.globalYMin, getMinY()
                - (newLengthY - oldLengthY) * bottomFraction);
        double newMaxY = Math.min(this.globalYMax, getMaxY()
                + (newLengthY - oldLengthY) * topFraction);
        setBoundsY(newMinY, newMaxY);

        final double leftFraction = (cValueX - xMin) / (xMax - xMin);
        final double rightFraction = 1.0 - leftFraction;

        double newMinX = Math.max(this.globalXMin, getMinX()
                - (newLengthX - oldLengthX) * leftFraction);
        double newMaxX = Math.min(this.globalXMax, getMaxX()
                + (newLengthX - oldLengthX) * rightFraction);

        //setIntervalX(newMinX, newMaxX);
        setBoundsX(newMinX, newMaxX);
    }

    /**
     * @param point
     */
    public Point2D getCoordinates(Point displayPoint) {

        Point2D sheetPoint = null;

        if (!getRulerBounds().contains(displayPoint)) {
            return sheetPoint;
        }

        sheetPoint = new Point2D.Double();

        double cValueX = getValueForScreenPositionX(displayPoint.x);
        double cValueY = getValueForScreenPositionY(displayPoint.y);

        cValueY = Math.rint(cValueY * 100.0) / 100.0;

        sheetPoint.setLocation(Math.floor(cValueX), cValueY);

        return sheetPoint;
    }
    
    
    public String getUnitName() {
        return jedule.getJeduleGrid().getUnitName();
    }
    
    public String getSlotsName() {
        return jedule.getJeduleGrid().getSlotsName();
    }

}
