/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.parser;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import net.sf.jedule.schedule.JeduleNode;
import net.sf.jedule.system.JeduleCluster;
import net.sf.jedule.system.JeduleGrid;
import net.sf.jedule.system.JeduleHost;
import net.sf.jedule.system.JeduleMeta;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * @author hunold
 *
 */
public class JeduleSimGridXmlParser implements JeduleXmlParser {

    private final static Logger LOGGER = Logger
            .getLogger(JeduleSimGridXmlParser.class.getName());

    private String fileName;
    private Document doc;
    
    public JeduleSimGridXmlParser() {
    }

    private Document initXmlDocument(final String fileName2)
            throws ParserConfigurationException, SAXException, IOException {
        LOGGER.fine("load XML document from " + fileName2);
        final DocumentBuilderFactory factory = DocumentBuilderFactory
                .newInstance();
        final DocumentBuilder builder = factory.newDocumentBuilder();
        final Document document = builder.parse(new File(fileName2));
        return document;
    }

    
    public void setInputFileName(String fileName) {
        this.fileName = fileName;
        this.doc = null;
    }
    
    
    /*
     * (non-Javadoc)
     * 
     * @see net.sf.jedule.parser.JeduleParser#readGrid()
     */
    public JeduleGrid readGrid() throws JeduleParserException {
        
        if( doc == null ) {
            try {
                doc = initXmlDocument(fileName);
            } catch (ParserConfigurationException e) {
                throw new JeduleParserException(e);
            } catch (SAXException e) {
                throw new JeduleParserException(e);
            } catch (IOException e) {
                throw new JeduleParserException(e);
            }
        } 
        
        
        return parseGrid(doc);
        
    }

    /*
     * (non-Javadoc)
     * 
     * @see net.sf.jedule.parser.JeduleParser#readNodes()
     */
    public List<JeduleNode> readNodes()
            throws JeduleParserException {
        
        final List<JeduleNode> nodeList = new ArrayList<JeduleNode>();
        
        if( doc == null ) {
            try {
                doc = initXmlDocument(fileName);
            } catch (ParserConfigurationException e) {
                throw new JeduleParserException(e);
            } catch (SAXException e) {
                throw new JeduleParserException(e);
            } catch (IOException e) {
                throw new JeduleParserException(e);
            }
        } 

        final NodeList nodeStatsList = doc
                .getElementsByTagName("node_statistics");
        for (int i = 0; i < nodeStatsList.getLength(); i++) {
            final Element statsNode = (Element) nodeStatsList.item(i);
            final List<JeduleNode> jNodes = readNodeElements(statsNode);
            for (final JeduleNode n : jNodes) {
                nodeList.add(n);
            }
        }

        return nodeList;
    }

    private List<JeduleNode> readNodeElements(Element statsNode) {
        final List<JeduleNode> nodes = new ArrayList<JeduleNode>();

        final String id = getStringValue(statsNode, "node_property", "id",
                "value");
        final String typeStr = getStringValue(statsNode, "node_property",
                "type", "value");
        final double startTime = getDoubleValue(statsNode, "node_property",
                "start_time", "value");
        final double endTime = getDoubleValue(statsNode, "node_property",
                "end_time", "value");

        if (typeStr == null) {
            throw new IllegalArgumentException("type of node is null");
        }

        final int configNb = statsNode.getElementsByTagName("configuration")
                .getLength();
        final NodeList configList = statsNode
                .getElementsByTagName("configuration");

        for (int c = 0; c < configNb; c++) {

            final Element configElem = (Element) configList.item(c);
            final int cluster_id = getIntegerValue(configElem, "conf_property",
                    "cluster_id", "value");

            final List<JeduleHost> hostList = new ArrayList<JeduleHost>();
            final Element hostListElem = (Element) configElem
                    .getElementsByTagName("host_lists").item(0);
            final NodeList list = hostListElem.getElementsByTagName("hosts");
            for (int i = 0; i < list.getLength(); i++) {
                final Element hostsElem = (Element) list.item(i);
                final int start_hostId = Integer.parseInt(hostsElem
                        .getAttribute("start"));
                final int host_nb = Integer.parseInt(hostsElem
                        .getAttribute("nb"));
                for (int hid = 0; hid < host_nb; hid++) {
                    hostList.add(new JeduleHost(start_hostId + hid));
                }
            }

            // merge hosts lists if two configurations in the same cluster
            // if not create a new jedule node
            boolean merged = false;
            for (final JeduleNode jN : nodes) {
                if (jN.getClusterId() == cluster_id) {
                    for (final JeduleHost h : hostList) {
                        jN.addHost(h);
                    }
                    merged = true;
                    break;
                }
            }

            if (!merged) {
                nodes.add(new JeduleNode(id, startTime, endTime, cluster_id,
                        hostList, typeStr));
            }
        }

        return nodes;
    }

    private int getIntegerValue(Element statsNode, String elemName, String key,
            String value) {
        return Integer
                .parseInt(getStringValue(statsNode, elemName, key, value));
    }

    private double getDoubleValue(Element statsNode, String elemName,
            String key, String value) {
        return Double.parseDouble(getStringValue(statsNode, elemName, key,
                value));
    }

    private String getStringValue(Element statsNode, String elemName,
            String key, final String value) {
        String retValue = null;
        final NodeList l = statsNode.getElementsByTagName(elemName);
        for (int i = 0; i < l.getLength(); i++) {
            final Element e = (Element) l.item(i);
            if (e.getAttribute("name").equals(key)) {
                retValue = e.getAttribute("value");
            }
        }
        return retValue;
    }

    private JeduleGrid parseGrid(Document document)
            throws JeduleParserException {
        final JeduleGrid grid = new JeduleGrid();
        
        
        final NodeList infoList = document.getElementsByTagName("info");
        for (int i = 0; i < infoList.getLength(); i++) {
            final Element metaNode = (Element) infoList.item(i);
            final String key   = metaNode.getAttribute("name");
            final String value = metaNode.getAttribute("value");
            if( key.equals("unit") ) {
                grid.setUnitName(value);
            } else if( key.equals("slots") ) {
                grid.setSlotsName(value);
            }
        }
        
        final NodeList clusterList = document.getElementsByTagName("cluster");

        if (clusterList.getLength() < 1) {
            throw new JeduleParserException("no cluster found in input file");
        }

        for (int i = 0; i < clusterList.getLength(); i++) {
            final Element elem = (Element) clusterList.item(i);
            final int host_nb = Integer.parseInt(elem.getAttribute("hosts"));
            final int first_host = Integer.parseInt(elem
                    .getAttribute("first_host"));

            final JeduleCluster cluster = new JeduleCluster(first_host);
            for (int host = 0; host < host_nb; host++) {
                cluster.addJeduleHost(new JeduleHost(host + first_host));
            }
            grid.addJeduleCluster(cluster);
        }
        return grid;
    }

    /*
     * (non-Javadoc)
     * 
     * @see net.sf.jedule.parser.JeduleParser#readMetaInfo(java.lang.String)
     */
    public JeduleMeta readMetaInfo()
            throws JeduleParserException {
        final JeduleMeta metaInfo = new JeduleMeta();

        if( doc == null ) {
            try {
                doc = initXmlDocument(fileName);
            } catch (ParserConfigurationException e) {
                throw new JeduleParserException(e);
            } catch (SAXException e) {
                throw new JeduleParserException(e);
            } catch (IOException e) {
                throw new JeduleParserException(e);
            }
        } 

        final NodeList metaList = doc.getElementsByTagName("meta");
        for (int i = 0; i < metaList.getLength(); i++) {
            final Element metaNode = (Element) metaList.item(i);

            final String key = metaNode.getAttribute("name");
            final String value = metaNode.getAttribute("value");

            metaInfo.addMeta(key, value);
        }

        return metaInfo;
    }


}
