/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.settings.global;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;
import java.util.logging.Logger;

import javax.xml.parsers.ParserConfigurationException;

import net.sf.jedule.JeduleOptions;
import net.sf.jedule.settings.global.parser.ColorMapParser;

import org.xml.sax.SAXException;

/**
 * @author sascha
 *
 */
public class JeduleSettings {

    private final static Logger LOGGER = Logger.getLogger(JeduleSettings.class.getName());
    private static JeduleSettings settings;

    private final static String jeduleConfigDir = ".jedule";

    private final static List<String> includedColorMapNames = new ArrayList<String>(
            Arrays.asList( new String[] { "conf1.xml" } ) );

    private final JeduleColorConfig colorConfig = new JeduleColorConfig();

    public static JeduleSettings getInstance() {
        if( settings == null ) {
            settings = new JeduleSettings();
        }
        return settings;
    }

    private JeduleSettings() {
    }

    public void init(final JeduleOptions jeduleOptions) {
        try {
            readColorConfig(jeduleOptions);
            
            
        } catch (JeduleSettingsException e) {
            e.printStackTrace();
        }
    }

    public JeduleColorConfig getJeduleColorConfig() {
        return colorConfig;
    }

    private void readColorConfig(JeduleOptions jeduleOptions) throws JeduleSettingsException {

        ColorMapParser parser = new ColorMapParser();

        for(String mapFileName : includedColorMapNames ) {
            try {
                final JeduleColorMap cMap = parser.readJeduleColorMap(JeduleSettings.class.getResourceAsStream("maps/"+mapFileName));
                colorConfig.addJeduleColorMap(cMap);
                LOGGER.fine("adding color map " + cMap.getName());
            } catch (ParserConfigurationException e) {
                throw new JeduleSettingsException(e);
            } catch (SAXException e) {
                throw new JeduleSettingsException(e);
            } catch (IOException e) {
                throw new JeduleSettingsException(e);
            }
        }

        final File userMapsDir = new File( getColorMapsDirectory() );

        final String[] userMapNames = userMapsDir.list(new FilenameFilter() {
            public boolean accept(File dir, String name) {
                if( name.endsWith(".xml" ) ) {
                    return true;
                }
                return false;
            }
        });

        final List<String> cMapList = new ArrayList<String>();
        if( userMapNames != null ) {
            for(String mapFileName : userMapNames) {
                cMapList.add(userMapsDir + File.separator + mapFileName);
            }
        }

        for(final String mapFileName : cMapList) {
            try {
                final JeduleColorMap cMap = parser.readJeduleColorMap(mapFileName);
                colorConfig.addJeduleColorMap(cMap);
                LOGGER.fine("adding color map " + cMap.getName());
            } catch (ParserConfigurationException e) {
                throw new JeduleSettingsException(e);
            } catch (SAXException e) {
                throw new JeduleSettingsException(e);
            } catch (IOException e) {
                throw new JeduleSettingsException(e);
            }
        }

        
        if( jeduleOptions.getJeduleColorMapFile() != null ) {
            final String cMapFileName = jeduleOptions.getJeduleColorMapFile();
            try {
                final JeduleColorMap cMap = parser.readJeduleColorMap(cMapFileName);
                colorConfig.addJeduleColorMap(cMap);
                LOGGER.fine("adding color map " + cMap.getName());
                
                colorConfig.setActiveJeduleColorMap(cMap.getName());
            } catch (ParserConfigurationException e) {
                throw new JeduleSettingsException(e);
            } catch (SAXException e) {
                throw new JeduleSettingsException(e);
            } catch (IOException e) {
                throw new JeduleSettingsException(e);
            }
            
        } else {
            // else active color map by given name
            colorConfig.setActiveJeduleColorMap(jeduleOptions
                    .getJeduleColorMapName());
        }
        
        
    }


    public static String getJeduleConfigDir() {
        return System.getProperty("user.home") + File.separator + jeduleConfigDir;
    }

    public static String getColorMapsDirectory() {
        String path = getJeduleConfigDir() + File.separator + "colormaps";
        return path;
    }


    public static String getVersionString() {
        String version = "unknown";

        Properties props = new Properties();
        try {
            InputStream instream = JeduleSettings.class
                    .getResourceAsStream("/net/sf/jedule/jedule.properties");
            if( instream != null ) {
                props.load(instream);
                if( props.getProperty("project.version") != null ) {
                	version = props.getProperty("project.version");
                }
            } else {
                LOGGER.warning("jedule property file not found");
            }
        } catch(IOException e) {
            LOGGER.warning("cannot load jedule properties");
        }

        return version;
    }

}
