/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.settings.global.parser;

import java.awt.Color;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import net.sf.jedule.settings.global.JeduleColorMap;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * @author sascha
 *
 */
public class ColorMapParser {

    private final Logger LOGGER = Logger.getLogger(ColorMapParser.class.getName());

    public ColorMapParser() {

    }

    public JeduleColorMap readJeduleColorMap(final String fileName)
            throws ParserConfigurationException, SAXException, IOException {
        return readJeduleColorMap(new FileInputStream(fileName));
    }

    public JeduleColorMap readJeduleColorMap(final InputStream stream)
            throws ParserConfigurationException, SAXException, IOException {
        JeduleColorMap config = null;

        final DocumentBuilderFactory factory = DocumentBuilderFactory
                .newInstance();
        final DocumentBuilder builder = factory.newDocumentBuilder();
        final Document document = builder.parse(stream);

        config = readConfiguration(document);

        return config;
    }

    /**
     * @param document
     * @return
     */
    private JeduleColorMap readConfiguration(Document document)
            throws SAXException {
        JeduleColorMap config = null;

        final NodeList cMapElems = document.getElementsByTagName("cmap");
        if (cMapElems.getLength() < 0) {
            throw new SAXException("no cmap element found");
        }
        if (cMapElems.getLength() > 1) {
            throw new SAXException("only one cmap element expected");
        }
        final Element cMapElem = (Element) cMapElems.item(0);

        final String mapName = cMapElem.getAttribute("name");
        if (mapName.equals("")) {
            LOGGER.warning("color map without name attribute!");
            throw new IllegalArgumentException("color map without name attribute!");
        } else {
            config = new JeduleColorMap(mapName);

            final NodeList confList = cMapElem.getElementsByTagName("conf");
            for (int i = 0; i < confList.getLength(); i++) {
                final Element confElem = (Element) confList.item(i);
                final String name = confElem.getAttribute("name");
                final String value = confElem.getAttribute("value");
                addToConfiguration(config, name, value);
            }

            final NodeList taskList = cMapElem.getElementsByTagName("task");
            for (int i = 0; i < taskList.getLength(); i++) {
                final Element task = (Element) taskList.item(i);
                final String taskId = task.getAttribute("id");
                readColorsForTask(config, taskId, task);
            }
            
            final NodeList compositeList = cMapElem
                    .getElementsByTagName("composite");            
            for (int i = 0; i < compositeList.getLength(); i++) {
                final Element composite = (Element) compositeList.item(i);
                final List<String> taskIdList = new ArrayList<String>();
                final NodeList tList = composite.getElementsByTagName("task");
                for(int j = 0; j < tList.getLength(); j++) {
                    final Element task = (Element) tList.item(j);
                    final String taskId = task.getAttribute("id");
                    taskIdList.add(taskId);
                }
                final String taskId = getHashKeyForComposite(taskIdList);
                //LOGGER.warning("add color for composite : " + taskId);
                readColorsForTask(config, taskId, composite);
            }
            
            
        }

        return config;
    }

    private void addToConfiguration(final JeduleColorMap config,
            final String name, final String value) {

        if( name.equals("min_font_size_label") ) {
            config.setMinFontSizeLabels(Integer.parseInt(value));
        } else if( name.equals("font_size_label") ) {
            config.setFontSizeLabels(Integer.parseInt(value));
        } else if( name.equals("font_size_axes") ) {
            config.setFontSizeAxes(Integer.parseInt(value));
        } else {
            LOGGER.warning("cannot parse configuration : " + name);
        }
                
    }

    private void readColorsForTask(JeduleColorMap config, String taskId,
            Element colorParent) {

        final NodeList colorList = colorParent.getElementsByTagName("color");
        for (int i = 0; i < colorList.getLength(); i++) {
            final Element colElem = (Element) colorList.item(i);
            final String colorName = colElem.getAttribute("type");
            if (colorName.equals("")) {
                LOGGER.warning("color type undefined");
            } else {
                final String colorRGBString = colElem.getAttribute("rgb");
                if (colorRGBString.equals("")) {
                    LOGGER.warning("no rgb color for id " + colorName);
                } else {
                    int rgbValue = Integer.parseInt(colorRGBString, 16);
                    final Color col = new Color(rgbValue);
                    config.addColor(taskId, colorName, col);
                }
            }
        }
        
    }

    /**
     * 
     * takes a list of task ids that define a composite and
     * creates a hashkey (id) for this composite task
     * 
     * @param taskIdList
     * @return
     */
    public static String getHashKeyForComposite(final List<String> taskIdList) {
        final Set<String> uniqueIdSet = new HashSet<String>();
        for(String id : taskIdList) {
            uniqueIdSet.add(id);
        }
        
        if( uniqueIdSet.size() == 1 ) {
            return uniqueIdSet.iterator().next();
        }
        
        final String[] idList = uniqueIdSet.toArray(new String[0]);
        Arrays.sort(idList);
        
        final StringBuffer keyBuf = new StringBuffer();
        for(int i=0; i<idList.length; i++) {
            keyBuf.append(idList[i]);
            keyBuf.append("|");
        }
        
        return keyBuf.toString();
    }
    
}
