/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.ui.swing;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Logger;

import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import net.sf.jedule.graphics.GraphicsConstants;
import net.sf.jedule.graphics.composed.JeduleGraphic;
import net.sf.jedule.io.JeduleFileFilter;
import net.sf.jedule.schedule.Jedule;
import net.sf.jedule.schedule.JeduleNode;
import net.sf.jedule.settings.JeduleContextSupportedFormats;
import net.sf.jedule.settings.JeduleImageInformation;
import net.sf.jedule.settings.JeduleSaveInformation;
import net.sf.jedule.settings.JeduleContextSupportedFormats.FILE_TYPE;
import net.sf.jedule.ui.IJSheetErrorReporter;
import net.sf.jedule.ui.IJSheetGraphicsContext;
import net.sf.jedule.ui.event.IJSheetContextListener;
import net.sf.jedule.ui.event.JSheetContextEvent;
import net.sf.jedule.ui.swing.components.JeduleGraphicSwingComponent;

import com.lowagie.text.DocumentException;
import com.lowagie.text.FontFactory;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.DefaultFontMapper;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfTemplate;
import com.lowagie.text.pdf.PdfWriter;

/**
 * @author hunold
 *
 */
public class JSheetSwingContext implements IJSheetGraphicsContext, KeyListener {

    private Jedule jedule;
    private final List<IJSheetContextListener> listeners = new ArrayList<IJSheetContextListener>();
    private final JeduleSwingViewer viewer = new JeduleSwingViewer();
    private final HashMap<Integer, JeduleGraphicSwingComponent> cId2SwingMap = new HashMap<Integer, JeduleGraphicSwingComponent>();
    private final List<Integer> clusterIdList = new ArrayList<Integer>();
    private String lastDirectory;
    private final IJSheetErrorReporter errorReporter;

    private boolean isNormalized = true;

    private final static Logger LOGGER = Logger
            .getLogger(JSheetSwingContext.class.getName());

    public JSheetSwingContext() {
        viewer.addKeyListener(this);
        // viewer.addComponentListener(this);

        errorReporter = new JSheetSwingErrorReporter(viewer);
    }

    public void setJedule(final Jedule jedule) {
        this.jedule = jedule;

        cId2SwingMap.clear();
        clusterIdList.clear();

        for (int i = 0; i < jedule.getJeduleGrid().getNumberOfClusters(); i++) {
            clusterIdList.add(i);
        }

        viewer.setJeduleName(jedule.getName());
        viewer.setMetaInfo(jedule.getMetaInfo());
        viewer.setClusterNum(this.jedule.getJeduleGrid().getNumberOfClusters());
    }

    public Jedule getJedule() {
        return jedule;
    }

    public void enableCluster(final int clusterId, final boolean enabled) {
        if (enabled) {
            if (!clusterIdList.contains(clusterId)) {
                clusterIdList.add(clusterId);
            }
        } else {
            clusterIdList.remove((Integer) clusterId);
        }
        Collections.sort(clusterIdList);
        showVisibleClusters();
    }

    public void showClusters(List<Integer> pClusterIdList) {
        clusterIdList.clear();
        clusterIdList.addAll(pClusterIdList);
        Collections.sort(clusterIdList);
        showVisibleClusters();
    }

    protected void showVisibleClusters() {
        viewer.removeAllJedulePanes();
        for (Integer clusterId : clusterIdList) {
            JeduleGraphicSwingComponent jGSC = cId2SwingMap.get(clusterId);
            viewer.addJedulePane(jGSC, clusterId);
        }
        viewer.validate();
        for (Integer clusterId : clusterIdList) {
            JeduleGraphicSwingComponent jGSC = cId2SwingMap.get(clusterId);
            jGSC.resize();
        }
        viewer.repaint();
    }

    public void show(final int width, final int height) {
        final int clusterWidth = width / clusterIdList.size();

        for (Integer clusterId : clusterIdList) {
            initClusterGraphic(clusterId, clusterWidth, height);
        }
        doLayout();

        viewer.pack();
        if (!viewer.isVisible()) {
            viewer.setLocationRelativeTo(null);
            viewer.setVisible(true);
        }
    }

    public void update() {

        if (viewer != null) {
            viewer.removeAllJedulePanes();
            final Dimension d = viewer.getScheduleDimension();
            show(d.width, d.height);
        }

    }

    /**
     * @param clusterId
     * @param clusterWidth
     * @param height
     */
    private void initClusterGraphic(Integer clusterId, int clusterWidth,
            int height) {
        final JeduleGraphic jG = new JeduleGraphic(this.jedule, clusterId,
                clusterWidth, height);

        final JeduleGraphicSwingComponent jGSC = new JeduleGraphicSwingComponent(
                jG);
        jGSC.setPreferredSize(new Dimension(jG.getWidth(), jG.getHeight()));
        cId2SwingMap.put(clusterId, jGSC);
        viewer.addJedulePane(jGSC, clusterId);
    }

    public void close() {
        viewer.setVisible(false);
        viewer.dispose();
    }

    /*
     * (non-Javadoc)
     *
     * @see net.sf.jedule.ui.IJSheetGraphicsContext#addIJSheetContextListener(net.sf.jedule.ui.IJSheetContextListener)
     */
    public synchronized void addIJSheetContextListener(
            IJSheetContextListener listener) {
        listeners.add(listener);
        viewer.addIJSheetContextListener(listener);
    }

    /*
     * (non-Javadoc)
     *
     * @see net.sf.jedule.ui.IJSheetGraphicsContext#fireIJSheetContextEvent(net.sf.jedule.ui.IJSheetContextEvent)
     */
    public synchronized void fireIJSheetContextEvent(JSheetContextEvent e) {
        for (IJSheetContextListener l : listeners) {
            l.performContextEvent(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see net.sf.jedule.ui.IJSheetGraphicsContext#removeIJSheetContextListener(net.sf.jedule.ui.IJSheetContextListener)
     */
    public synchronized void removeIJSheetContextListener(
            IJSheetContextListener listener) {
        listeners.remove(listener);
        viewer.removeIJSheetContextListener(listener);
    }

    /*
     * (non-Javadoc)
     *
     * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
     */
    public void keyPressed(KeyEvent e) {
    }

    /*
     * (non-Javadoc)
     *
     * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
     */
    public void keyReleased(KeyEvent e) {
    }

    /*
     * (non-Javadoc)
     *
     * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
     */
    public void keyTyped(KeyEvent e) {
        final char keyChar = e.getKeyChar();

        // System.out.println(e);

        if (keyChar == 'r') {
            LOGGER.fine("reload graphics");
            fireIJSheetContextEvent(new JSheetContextEvent(this,
                    JSheetContextEvent.CMD_JEDULE_RELOAD, null));
        } else if (keyChar == 'q') {
            LOGGER.fine("quit");
            fireIJSheetContextEvent(new JSheetContextEvent(this,
                    JSheetContextEvent.CMD_JEDULE_QUIT, null));
        } else if (keyChar == 'b') {
            // isNormalized = false;
            doLayout();
        } else if (keyChar == 's') {

            // System.out.println(e.getModifiersEx());

            if ((e.getModifiersEx() & KeyEvent.ALT_DOWN_MASK) == KeyEvent.ALT_DOWN_MASK) {
                final String nodeName = JOptionPane.showInputDialog(viewer,
                        "Node name (regex)");
                if (nodeName != null) {
                    fireIJSheetContextEvent(new JSheetContextEvent(this,
                            JSheetContextEvent.MARK_NODE, nodeName));
                }
            } else {
                if (viewer != null) {
                    fireIJSheetContextEvent(new JSheetContextEvent(this,
                            JSheetContextEvent.MAKE_SNAPSHOT, null));
                }
            }

        } else if (keyChar == 'i') {
            LOGGER.fine("info");
            viewer.toggleInfoComponent();
        } else if (keyChar == 'n') {
            isNormalized = !isNormalized;
            doLayout();
        }

    }

    /*
     * (non-Javadoc)
     *
     * @see net.sf.jedule.ui.IJSheetGraphicsContext#showNodeInformation(java.util.List)
     */
    public void showNodeInformation(List<String> nodeIdList) {

        final List<JeduleNode> jNodeList = new ArrayList<JeduleNode>();
        for (String nodeId : nodeIdList) {
            JeduleNode n = jedule.getJeduleNode(nodeId);
            if (n != null) {
                jNodeList.add(n);
            } else {
                LOGGER.warning("cannot find node with id " + nodeId);
            }
        }
        if (jNodeList.size() > 0) {
            viewer.showNodeInformation(jNodeList);
        }

    }



    /* (non-Javadoc)
     * @see net.sf.jedule.ui.IJSheetGraphicsContext#getJeduleSaveInformation()
     */
    public JeduleSaveInformation getJeduleSaveInformation() {

        JeduleSaveInformation info = null;

        final JFileChooser fC = new JFileChooser();

        if (lastDirectory != null) {
            final File f = new File(lastDirectory);
            if (f.exists()) {
                fC.setCurrentDirectory(f);
            }
        }
        //fC.setAcceptAllFileFilterUsed(false);

        for (FileFilter ff : getJeduleContextSupportedFormats()
                .getSupportedExportFileFilters()) {
            fC.addChoosableFileFilter(ff);
        }

        fC.setDialogTitle("Save As");

        int option = fC.showSaveDialog(viewer);

        if (option == JFileChooser.APPROVE_OPTION) {

        	if( fC.getSelectedFile().isDirectory() ) {
        		LOGGER.warning("Cannot save image, directory selected!");
        		return null;
        	}


            String fileName = fC.getSelectedFile().toString();
            lastDirectory = fC.getSelectedFile().getPath();

            final FileFilter ff = fC.getFileFilter();

            FILE_TYPE fileType = null;
            for(JeduleFileFilter jFF : getJeduleContextSupportedFormats()
                    .getSupportedExportFileFilters()) {
                if( jFF.getDescription().equals(ff.getDescription()) ) {
                    fileType = jFF.getFileType();

                    if( ! fileName.endsWith(jFF.getSuffix()) ) {
                        if( jFF.getSuffix().startsWith(".")) {
                            fileName = fileName + jFF.getSuffix();
                        } else {
                            fileName = fileName + "." + jFF.getSuffix();
                        }
                    }

                    break;
                }
            }

            final JeduleSaveSettingsDialog dialog = new JeduleSaveSettingsDialog(viewer, true);
            dialog.pack();
            dialog.setImageDimension(viewer.getScheduleDimension().width,
                    viewer.getScheduleDimension().height);
            dialog.setLocationRelativeTo(viewer);
            dialog.setVisible(true);
            final JeduleImageInformation imageInfo = dialog.getJeduleImageInformation();
            if( imageInfo != null ) {
                info = new JeduleSaveInformation(fileName, fileType);
                info.setJeduleImageInformation(imageInfo);
            } else {
                info = null;
            }
        }

        return info;
    }

    /* (non-Javadoc)
     * @see net.sf.jedule.ui.IJSheetGraphicsContext#getFileName()
     */
    public String getFileName() {
        String fileName = null;

        final JFileChooser fC = new JFileChooser();

        if (lastDirectory != null) {
            final File f = new File(lastDirectory);
            if (f.exists()) {
                fC.setCurrentDirectory(f);
            }
        }

        int option = fC.showOpenDialog(viewer);

        if (option == JFileChooser.APPROVE_OPTION) {
            fileName = fC.getSelectedFile().toString();
            lastDirectory = fC.getSelectedFile().getPath();
        }

        return fileName;
    }

    public IJSheetErrorReporter getJSheetErrorReporter() {
        return errorReporter;
    }

    public void save(final JeduleSaveInformation saveSpec) throws IOException {
        viewer.removeAllJedulePanes();
        final int clusterWidth = saveSpec.getImageInfo().getImageWidth()
                / clusterIdList.size();
        for (Integer clusterId : clusterIdList) {
            initClusterGraphic(clusterId, clusterWidth, saveSpec.getImageInfo()
                    .getImageHeight());
        }
        viewer.pack();
        if (isNormalized) {
            doNormalizedLayout();
        }

        saveViewToFile(saveSpec.getFileName(), saveSpec.getFileType());
    }

    private void saveViewToFile(String filename, FILE_TYPE outputType)
            throws IOException {

        if( outputType == FILE_TYPE.PNG || outputType == FILE_TYPE.JPG ) {

            String formatName = "PNG";
            if( outputType == FILE_TYPE.JPG  ) {
                formatName = "JPEG";
            }

            final BufferedImage bi = new BufferedImage(viewer.getScheduleDimension()
                    .width, viewer.getScheduleDimension().height,
                    BufferedImage.TYPE_INT_RGB);
            final Graphics2D g = bi.createGraphics();

            viewer.drawGraphics(g);

            g.dispose();

            ImageIO.write(bi, formatName, new File(filename));

        } else if( outputType == FILE_TYPE.PDF ) {

            try {

                final int w = viewer.getScheduleDimension().width;
                final int h = viewer.getScheduleDimension().height;

                final com.lowagie.text.Document document = new com.lowagie.text.Document(
                        new Rectangle(w, h));

                final DefaultFontMapper mapper = new DefaultFontMapper();
                FontFactory.registerDirectories();

                PdfWriter writer = PdfWriter.getInstance(document,
                        new FileOutputStream(filename));

                document.open();

                PdfContentByte cb = writer.getDirectContent();
                PdfTemplate tp = cb.createTemplate(w, h);

                Graphics2D g2 = tp.createGraphics(w, h, mapper);

                tp.setWidth(w);
                tp.setHeight(h);

                viewer.drawGraphics(g2);

                g2.dispose();

                cb.addTemplate(tp, 0, 0);

                document.close();

            } catch(DocumentException e) {
                throw new IOException(e);
            }

//        } else if( outputType == FILE_TYPE.TIKZ ) {
//
//            final int w = viewer.getScheduleDimension().width;
//            final int h = viewer.getScheduleDimension().height;
//        	
//            
//            FileOutputStream os = new FileOutputStream(filename);
//			TikzGraphics2D dev = new TikzGraphics2D();
//            
//			viewer.drawGraphics(dev);
//			
//			dev.close();
//			os.close();
			        	
        } else {
        	LOGGER.severe("unknown output type " + outputType);
        }
        
        
    }

    /* (non-Javadoc)
     * @see net.sf.jedule.ui.IJSheetGraphicsContext#makeSnapshot()
     */
    public void makeSnapshot() throws IOException {
        saveViewToFile(GraphicsConstants.SNAPSHOT_NAME, FILE_TYPE.PNG);
    }



    /*
     * (non-Javadoc)
     *
     * @see net.sf.jedule.ui.IJSheetGraphicsContext#normalize()
     */
    public void normalize() {
        LOGGER.fine("normalize time axis");
        isNormalized = true;
    }

    protected void doLayout() {
        if (isNormalized) {
            doNormalizedLayout();
        } else {
            doStandardLayout();
        }
    }

    protected void doNormalizedLayout() {
        double yMax = 0;
        for (int i = 0; i < jedule.getJeduleGrid().getNumberOfClusters(); i++) {
            if (jedule.getFinishTime(i) > yMax) {
                yMax = jedule.getFinishTime(i);
            }
        }

        LOGGER.fine("y max : " + yMax);
        for (Integer clusterId : cId2SwingMap.keySet()) {
            JeduleGraphicSwingComponent jGSC = cId2SwingMap.get(clusterId);
            jGSC.resetView();
            jGSC.setBoundsY(0.0, yMax);
            jGSC.repaint();
        }
    }

    protected void doStandardLayout() {
        for (Integer clusterId : cId2SwingMap.keySet()) {
            JeduleGraphicSwingComponent jGSC = cId2SwingMap.get(clusterId);
            jGSC.resetView();
            jGSC.doLayout();
            jGSC.repaint();
        }
    }

    /* (non-Javadoc)
     * @see net.sf.jedule.ui.IJSheetGraphicsContext#markNodes(java.util.List)
     */
    public void markNodes(List<String> nodeIdList) {

        LOGGER.warning("NOT IMPLEMENTED YET");

    }

    /* (non-Javadoc)
     * @see net.sf.jedule.ui.IJSheetGraphicsContext#getJeduleContextSupportedFormats()
     */
    public JeduleContextSupportedFormats getJeduleContextSupportedFormats() {
        return new SwingContextSupportedFormats();
    }

    /* (non-Javadoc)
     * @see net.sf.jedule.ui.IJSheetGraphicsContext#showErrorDialog(java.lang.String)
     */
    public void showErrorDialog(String message) {
        JOptionPane.showConfirmDialog(viewer, message, "Error",
                JOptionPane.ERROR_MESSAGE);
    }

}
