/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.ui.swing;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

import net.sf.jedule.schedule.JeduleNode;
import net.sf.jedule.settings.global.JeduleSettings;
import net.sf.jedule.system.JeduleMeta;
import net.sf.jedule.ui.event.IJSheetContextEventObject;
import net.sf.jedule.ui.event.IJSheetContextListener;
import net.sf.jedule.ui.event.JSheetClusterSelectionEvent;
import net.sf.jedule.ui.event.JSheetClusterSelectionEvent2;
import net.sf.jedule.ui.event.JSheetContextEvent;
import net.sf.jedule.ui.event.JSheetLoadContextEvent;
import net.sf.jedule.ui.swing.components.JeduleGraphicSwingComponent;

/**
 * @author hunold
 *
 */
public class JeduleSwingViewer extends JFrame implements
        IJSheetContextEventObject {

     private final static Logger LOGGER = Logger
            .getLogger(JeduleSwingViewer.class.getName());

    private final MetaInfoPane metaInfoPane = new MetaInfoPane();
    private final JPanel jedulePane = new JPanel();
    private final JeduleMenuBar menuBar = new JeduleMenuBar(this);
    private final List<IJSheetContextListener> contextListeners = new ArrayList<IJSheetContextListener>();

    private boolean infoComponentVisible = false;
    private final JeduleSwingInfoPanel infoPanel = new JeduleSwingInfoPanel();

    private boolean[] visibleClusterAr;

    public JeduleSwingViewer() {
        initFrame();
    }

    private void initFrame() {
        setTitle("Jedule");
        jedulePane.setLayout(new BoxLayout(jedulePane, BoxLayout.X_AXIS));
        getContentPane().add(metaInfoPane, BorderLayout.NORTH);
        getContentPane().add(jedulePane, BorderLayout.CENTER);
        setJMenuBar(menuBar);
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
    }

    public void addJedulePane(
            final JeduleGraphicSwingComponent clusterComponent,
            final int globalId) {
        jedulePane.add(clusterComponent);
        visibleClusterAr[globalId] = true;
        updateMenu();
        for (final IJSheetContextListener l : contextListeners) {
            clusterComponent.addIJSheetContextListener(l);
        }
    }

    public void removeJedulePane(
            final JeduleGraphicSwingComponent clusterComponent,
            final int globalId) {
        jedulePane.remove(clusterComponent);
        visibleClusterAr[globalId] = false;
        updateMenu();
    }

    public void removeAllJedulePanes() {
        jedulePane.removeAll();
        for (int i = 0; i < visibleClusterAr.length; i++) {
            visibleClusterAr[i] = false;
        }
    }

    /**
     *
     * @return returns the size of the panel which contains the drawn schedule
     */
    public Dimension getScheduleDimension() {
        return jedulePane.getSize();
    }

    public void setJeduleName(final String name) {
        setTitle(name);
    }

    public void setClusterNum(final int clusterNum) {
        menuBar.setClusterNum(clusterNum);
        visibleClusterAr = new boolean[clusterNum];
    }

    public void drawGraphics(Graphics g) {
        //getContentPane().paint(g);
        jedulePane.paint(g);
    }

    /*
     * (non-Javadoc)
     *
     * @see net.sf.jedule.ui.IJSheetContextEventObject#addIJSheetContextListener(net.sf.jedule.ui.IJSheetContextListener)
     */
    public synchronized void addIJSheetContextListener(
            final IJSheetContextListener listener) {
        contextListeners.add(listener);
        menuBar.addIJSheetContextListener(listener);
    }

    /*
     * (non-Javadoc)
     *
     * @see net.sf.jedule.ui.IJSheetContextEventObject#fireIJSheetContextEvent(net.sf.jedule.ui.JSheetContextEvent)
     */
    public synchronized void fireIJSheetContextEvent(final JSheetContextEvent e) {
        for (final IJSheetContextListener l : contextListeners) {
            l.performContextEvent(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see net.sf.jedule.ui.IJSheetContextEventObject#removeIJSheetContextListener(net.sf.jedule.ui.IJSheetContextListener)
     */
    public synchronized void removeIJSheetContextListener(
            final IJSheetContextListener listener) {
        contextListeners.remove(listener);
        menuBar.removeIJSheetContextListener(listener);
    }

    public void toggleInfoComponent() {
        infoComponentVisible = !infoComponentVisible;
        if (infoComponentVisible) {
            System.out.println("add info panel");
            infoPanel.setPreferredSize(new Dimension(getWidth(), 50));
            getContentPane().add(infoPanel, BorderLayout.SOUTH);
        } else {
            getContentPane().remove(infoPanel);
        }
        getContentPane().validate();
        getContentPane().repaint();
    }

    public void showNodeInformation(final List<JeduleNode> nodeList) {
        infoPanel.showJeduleNodes(nodeList);
        infoPanel.validate();
        infoPanel.repaint();
    }

    private void updateMenu() {
        for (int i = 0; i < visibleClusterAr.length; i++) {
            // System.out.println("updateMenu " + visibleClusterAr[i]);
            menuBar.setClusterEnabled(i, visibleClusterAr[i]);
        }
    }

    static class JeduleMenuBar extends JMenuBar implements ActionListener,
            IJSheetContextEventObject {

        private final JFrame parent;

        private final JMenu clusterMenu = new JMenu("Show clusters");

        private final List<IJSheetContextListener> contextListeners = new ArrayList<IJSheetContextListener>();

        private int clusterNum = 0;

        private final List<JMenuItem> clusterItemList = new ArrayList<JMenuItem>();

        public JeduleMenuBar(final JFrame parent) {
            this.parent = parent;
            initMenus();
        }

        private void initMenus() {
            add(createFileMenu());
            add(createOptionMenu());
            add(createHelpMenu());
        }

        private JMenu createHelpMenu() {
            final JMenu menu = new JMenu("Help");
            final JMenuItem aboutItem = new JMenuItem("About");
            aboutItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            JeduleAboutDialog d = new JeduleAboutDialog(parent);
                            d.setVisible(true);
                        }
                    });
                }
            });

            menu.add(aboutItem);

            return menu;
        }

        private JMenu createFileMenu() {
            final JMenu menu = new JMenu("File");

            final JMenuItem openSimGridItem = new JMenuItem(
                    "Load SimGrid file..");
            menu.add(openSimGridItem);
            openSimGridItem.addActionListener(this);
            openSimGridItem.setActionCommand("open_simgrid");

            final JMenuItem openGenMtsItem = new JMenuItem("Load genMTS file..");
            menu.add(openGenMtsItem);
            openGenMtsItem.addActionListener(this);
            openGenMtsItem.setActionCommand("open_genmts");

            final JMenuItem saveItem = new JMenuItem("Save As..");
            menu.add(saveItem);
            saveItem.addActionListener(this);
            saveItem.setActionCommand("save_as");

            return menu;
        }

        private JMenu createOptionMenu() {
            final JMenu menu = new JMenu("Options");
            menu.add(clusterMenu);

            final JMenuItem clustSelItem = new JMenuItem("Select cluster");
            clustSelItem.setActionCommand("select_clusters");
            clustSelItem.addActionListener(this);
            menu.add(clustSelItem);

            final JMenu colorMapMenu = new JMenu("ColorMaps");
            menu.add(colorMapMenu);

            final ButtonGroup cMapGroup = new ButtonGroup();

            for (String colorMapName : JeduleSettings.getInstance()
                    .getJeduleColorConfig().getJeduleColorMapNames()) {
                JRadioButtonMenuItem cMapButton = new JRadioButtonMenuItem(colorMapName);
                cMapGroup.add(cMapButton);
                cMapButton.setActionCommand("COLORMAP_ACTION");
                cMapButton.addActionListener(this);
                if (colorMapName.equals(JeduleSettings.getInstance()
                        .getJeduleColorConfig().getActiveJeduleColorMap()
                        .getName())) {
                    cMapButton.setSelected(true);
                }
                colorMapMenu.add(cMapButton);
            }

            return menu;
        }

        public void setClusterNum(final int clusterNum) {
            this.clusterNum = clusterNum;
            clusterMenu.removeAll();
            clusterItemList.clear();

            for (int i = 0; i < clusterNum; i++) {
                final JMenuItem clusterItem = new JCheckBoxMenuItem(Integer
                        .toString(i), true);
                clusterItem.addActionListener(this);
                clusterItem.setActionCommand(Integer.toString(i));
                clusterItemList.add(clusterItem);
                clusterMenu.add(clusterItem);
            }
        }

        public void setClusterEnabled(final int clusterId, final boolean enabled) {
            clusterItemList.get(clusterId).setSelected(enabled);
        }

        /*
         * (non-Javadoc)
         *
         * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
         */
        public void actionPerformed(final ActionEvent e) {
            if (e.getSource() instanceof JCheckBoxMenuItem) {
                final String cmd = e.getActionCommand();
                final int clusterId = Integer.parseInt(cmd);
                final boolean enabled = ((JCheckBoxMenuItem) (e.getSource()))
                        .getState();
                fireIJSheetContextEvent(new JSheetClusterSelectionEvent(this,
                        clusterId, enabled));
            } else {
                final String cmd = e.getActionCommand();
                if (cmd.equals("open_simgrid")) {
                    fireIJSheetContextEvent(new JSheetLoadContextEvent(this,
                            "load", "simgrid"));
                } else if (cmd.equals("open_genmts")) {
                    fireIJSheetContextEvent(new JSheetLoadContextEvent(this,
                            "load", "genmts"));
                } else if (cmd.equals("select_clusters")) {
                    final JeduleClusterSelection selector = new JeduleClusterSelection(
                            this.clusterNum);
                    if (selector.select(parent)) {
                        final List<Integer> idList = selector.getSelectedIds();
                        if (idList.size() > 0) {
                            fireIJSheetContextEvent(new JSheetClusterSelectionEvent2(
                                    this, idList));
                        }
                    }
                } else if( cmd.equals("COLORMAP_ACTION")) {

                    final JRadioButtonMenuItem button = (JRadioButtonMenuItem) e
                            .getSource();
                    final String activeMapName = button.getText();
                    fireIJSheetContextEvent(new JSheetContextEvent(this,
                            JSheetContextEvent.CMD_JEDULE_CMAP_CHANGED, activeMapName));

                } else if( cmd.equals("save_as")) {

                    fireIJSheetContextEvent(new JSheetContextEvent(this,
                            JSheetContextEvent.SAVE_IMAGE, null));

                } else {
                    LOGGER.warning("unhandled action command " + cmd);
                }
            }
        }

        /*
         * (non-Javadoc)
         *
         * @see net.sf.jedule.ui.IJSheetContextEventObject#addIJSheetContextListener(net.sf.jedule.ui.IJSheetContextListener)
         */
        public synchronized void addIJSheetContextListener(
                final IJSheetContextListener listener) {
            contextListeners.add(listener);
        }

        /*
         * (non-Javadoc)
         *
         * @see net.sf.jedule.ui.IJSheetContextEventObject#fireIJSheetContextEvent(net.sf.jedule.ui.JSheetContextEvent)
         */
        public synchronized void fireIJSheetContextEvent(
                final JSheetContextEvent e) {
            for (final IJSheetContextListener l : contextListeners) {
                l.performContextEvent(e);
            }
        }

        /*
         * (non-Javadoc)
         *
         * @see net.sf.jedule.ui.IJSheetContextEventObject#removeIJSheetContextListener(net.sf.jedule.ui.IJSheetContextListener)
         */
        public synchronized void removeIJSheetContextListener(
                final IJSheetContextListener listener) {
            contextListeners.remove(listener);
        }

    }

    /**
     * @param metaInfo
     */
    public void setMetaInfo(final JeduleMeta metaInfo) {
        metaInfoPane.setMetaInfo(metaInfo);
    }

    static class MetaInfoPane extends JPanel {

        private JeduleMeta metaInfo;
        private String metaString = "";
        private final JLabel metaLabel = new JLabel();

        public MetaInfoPane() {
            initUI();
            writeMetaInfo();
        }

        private void initUI() {
            metaLabel.setHorizontalTextPosition(SwingConstants.LEFT);
            setBackground(Color.WHITE);
            setLayout(new GridLayout(1, 1));
            add(metaLabel, BorderLayout.CENTER);
        }

        public void setMetaInfo(final JeduleMeta metaInfo) {
            this.metaInfo = metaInfo;
            writeMetaInfo();
            repaint();
        }

        private void writeMetaInfo() {
            final StringBuffer metaBuffer = new StringBuffer();
            metaBuffer.append("|");
            if (metaInfo != null) {
                for (final String key : metaInfo.getKeys()) {
                    final String value = metaInfo.getMeta(key);
                    metaBuffer.append(key);
                    metaBuffer.append(":");
                    metaBuffer.append(value);
                    metaBuffer.append("|");
                }
            }
            metaString = metaBuffer.toString();
            metaLabel.setText(metaString);
        }

    }

}
