/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.ui.swing.components;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Logger;

import javax.swing.JComponent;
import javax.swing.SwingUtilities;

import net.sf.jedule.graphics.JSheetObject;
import net.sf.jedule.ui.IJSheetGraphicsContextComponent;
import net.sf.jedule.ui.event.IJSheetContextEventObject;
import net.sf.jedule.ui.event.IJSheetContextListener;
import net.sf.jedule.ui.event.JSheetContextEvent;
import net.sf.jedule.ui.swing.JSheetSwingComponentFactory;

/**
 * @author hunold
 *
 */
public abstract class JSheetSwingComponent extends JComponent implements
        IJSheetGraphicsContextComponent, MouseListener, MouseMotionListener,
        MouseWheelListener, IJSheetContextEventObject {

    private static final long serialVersionUID = 1L;

    private final static Logger LOGGER = Logger
            .getLogger(JSheetSwingComponent.class.getName());

    private final HashMap<JSheetObject, JSheetSwingComponent> sheet2swingMap = new HashMap<JSheetObject, JSheetSwingComponent>();
    private final JSheetObject sheetObject;

    private final List<MouseListener> mouseListeners = new ArrayList<MouseListener>();
    private final List<MouseWheelListener> mouseWheelListeners = new ArrayList<MouseWheelListener>();
    private final List<IJSheetContextListener> contextListeners = new ArrayList<IJSheetContextListener>();

    public JSheetSwingComponent(JSheetObject sheetObject) {
        this.sheetObject = sheetObject;
        addMouseListener(this);
        addMouseMotionListener(this);
        addMouseWheelListener(this);
        setDoubleBuffered(false);
        setOpaque(true);
    }

    public abstract void renderComponent(Graphics2D g);

    /*
     * (non-Javadoc)
     *
     * @see net.sf.jedule.ui.IJSheetContextEventObject#addIJSheetContextListener(net.sf.jedule.ui.IJSheetContextListener)
     */
    public synchronized void addIJSheetContextListener(
            IJSheetContextListener listener) {
        contextListeners.add(listener);
    }

    /*
     * (non-Javadoc)
     *
     * @see net.sf.jedule.ui.IJSheetContextEventObject#fireIJSheetContextEvent(net.sf.jedule.ui.JSheetContextEvent)
     */
    public synchronized void fireIJSheetContextEvent(JSheetContextEvent e) {
        for (IJSheetContextListener l : contextListeners) {
            l.performContextEvent(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see net.sf.jedule.ui.IJSheetContextEventObject#removeIJSheetContextListener(net.sf.jedule.ui.IJSheetContextListener)
     */
    public synchronized void removeIJSheetContextListener(
            IJSheetContextListener listener) {
        contextListeners.remove(listener);
    }

    public void addParentMouseListener(final MouseListener l) {
        mouseListeners.add(l);
    }

    public void removeParentMouseListener(final MouseListener l) {
        mouseListeners.remove(l);
    }

    public void addParentMouseWheelListener(final MouseWheelListener l) {
        mouseWheelListeners.add(l);
    }

    public void removeParentMouseWheelListener(final MouseWheelListener l) {
        mouseWheelListeners.remove(l);
    }

    protected JSheetObject getJSheetObject() {
        return sheetObject;
    }

    /*
     * (non-Javadoc)
     *
     * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
     */
    @Override
    protected void paintComponent(Graphics g) {
        LOGGER.fine("paint component in " + this);
        super.paintComponent(g);
        renderComponent((Graphics2D) g);
    }

    public void doLayout() {
        JSheetSwingComponentFactory swingFact = new JSheetSwingComponentFactory();
        JSheetObject sheetObject = getJSheetObject();
        sheetObject.doLayout();
        
        for (JSheetObject child : sheetObject.getChildObjects()) {

            if (!sheet2swingMap.containsKey(child)) {
                JSheetSwingComponent swingC = (JSheetSwingComponent) swingFact
                        .getIJSheetGraphicsContextComponent(child);
                swingC.addParentMouseListener(this);
                swingC.addParentMouseWheelListener(this);
                for (IJSheetContextListener l : contextListeners) {
                    swingC.addIJSheetContextListener(l);
                }
                sheet2swingMap.put(child, swingC);
                add(swingC);
            }

            JSheetSwingComponent swingComponent = sheet2swingMap.get(child);
            if( child.isVisable() ) {
                final Point childPos = sheetObject.getChildPosition(child);
                swingComponent.setBounds(childPos.x, childPos.y, child.getWidth(),
                        child.getHeight());
                swingComponent.doLayout();
                swingComponent.setVisible(true);
            }  else {
                swingComponent.setVisible(false);
            }
        }

    }

    public final void mouseClicked(MouseEvent e) {
        customMouseClicked(e);
        dispatchMouseEvent(e);
    }

    public final void mouseEntered(MouseEvent e) {
        customMouseEntered(e);
        dispatchMouseEvent(e);
    }

    public final void mouseExited(MouseEvent e) {
        customMouseExited(e);
        dispatchMouseEvent(e);
    }

    public final void mousePressed(MouseEvent e) {
        customMousePressed(e);
        dispatchMouseEvent(e);
    }

    public final void mouseReleased(MouseEvent e) {
        customMouseReleased(e);
        dispatchMouseEvent(e);
    }

    public final void mouseDragged(MouseEvent e) {
        customMouseDragged(e);
        dispatchMouseEvent(e);
    }

    public final void mouseMoved(MouseEvent e) {
        customMouseMoved(e);
        dispatchMouseEvent(e);
    }

    public void mouseWheelMoved(MouseWheelEvent e) {
        customMouseWheelMoved(e);
        dispatchMouseWheelEvent(e);
    }

    private void dispatchMouseEvent(final MouseEvent e) {
        for (MouseListener l : mouseListeners) {
            if (l instanceof JComponent) {
                final JComponent targetComponent = (JComponent) l;
                final Point targetPoint = SwingUtilities.convertPoint(this, e
                        .getPoint(), targetComponent);
                targetComponent.dispatchEvent(new MouseEvent(targetComponent, e
                        .getID(), e.getWhen(), e.getModifiers(), targetPoint.x,
                        targetPoint.y, e.getClickCount(), e.isPopupTrigger()));

            }
        }
    }

    private void dispatchMouseWheelEvent(final MouseWheelEvent e) {
        for (MouseWheelListener l : mouseWheelListeners) {
            if (l instanceof JComponent) {
                final JComponent targetComponent = (JComponent) l;
                final Point targetPoint = SwingUtilities.convertPoint(this, e
                        .getPoint(), targetComponent);
                targetComponent.dispatchEvent(new MouseWheelEvent(
                        targetComponent, e.getID(), e.getWhen(), e
                                .getModifiers(), targetPoint.x, targetPoint.y,
                        e.getClickCount(), e.isPopupTrigger(), e
                                .getScrollType(), e.getScrollAmount(), e
                                .getWheelRotation()));

            }
        }
    }

    public void customMouseClicked(MouseEvent e) {
    }

    public void customMouseEntered(MouseEvent e) {
    }

    public void customMouseExited(MouseEvent e) {
    }

    public void customMousePressed(MouseEvent e) {
    }

    public void customMouseReleased(MouseEvent e) {
    }

    public void customMouseDragged(MouseEvent e) {
    }

    public void customMouseMoved(MouseEvent e) {
    }

    public void customMouseWheelMoved(MouseWheelEvent e) {
    }

}
