/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.ui.swing.components;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.util.logging.Logger;

import javax.swing.BorderFactory;

import net.sf.jedule.graphics.composed.JSheetRuler;
import net.sf.jedule.settings.global.JeduleColorMap;
import net.sf.jedule.settings.global.JeduleSettings;
import net.sf.jedule.ui.swing.components.SwingArrowObject.ARRAY_TYPE;
import net.sf.jedule.ui.tools.ScreenTools;

/**
 * @author sascha
 *
 */
public class JSheetSwingRuler extends JSheetSwingComponent {
    
    private static final long serialVersionUID = 1L;
    private final static Logger LOGGER = Logger
            .getLogger(JSheetSwingRuler.class.getName());
    private final JSheetRuler ruler;

    private final Stroke gridStroke = new BasicStroke(1.0f,
            BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 10.0f, new float[] {
                    3.0f, 6.0f }, 0.0f);

    public JSheetSwingRuler(JSheetRuler sheetRuler) {
        super(sheetRuler);
        this.ruler = sheetRuler;
        // setBorder(BorderFactory.createLineBorder(Color.BLACK));
        setBorder(BorderFactory.createEtchedBorder());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * net.sf.jedule.ui.swing.IJSheetSwingComponent#renderComponent(java.awt
     * .Graphics2D)
     */
    public void renderComponent(Graphics2D g) {

        LOGGER.fine("renderComponent in SwingRuler, size : " + ruler.getSize());

        g.setColor(Color.WHITE);
        g.fillRect(0, 0, getWidth() - 1, getHeight() - 1);

        final int xAxisL = ruler.getWidth() - ruler.getLeftOffset()
                - ruler.getRightOffset();
        final int xAxisStartX = ruler.getLeftOffset();
        final int xAxisEndX = ruler.getWidth() - ruler.getRightOffset();
        final int xAxisY = ruler.getHeight() - ruler.getBottomOffset();

        final int yAxisL = ruler.getHeight() - ruler.getTopOffset()
                - ruler.getBottomOffset();
        final int yAxisStartY = ruler.getHeight() - ruler.getBottomOffset();
        final int yAxisEndY = ruler.getTopOffset();
        final int yAxisX = ruler.getLeftOffset();

        
        final JeduleColorMap colorMap = JeduleSettings.getInstance()
                .getJeduleColorConfig().getActiveJeduleColorMap();
        g.setFont(new Font("Monospaced", Font.PLAIN, colorMap
                        .getFontSizeAxes()));
        g.setColor(Color.BLACK);

        g.drawLine(xAxisStartX, xAxisY, xAxisEndX, xAxisY);
        final String procString = ruler.getSlotsName();

        final Point procPoint = ScreenTools.getStringPosition(g, procString,
                new Dimension(xAxisL, ruler.getBottomOffset()));

        g.drawString(procString, procPoint.x + ruler.getLeftOffset(), ruler
                .getHeight()
                - procPoint.y + g.getFont().getSize());

        g.drawLine(yAxisX, yAxisEndY, yAxisX, yAxisStartY);

        AffineTransform saveAT = g.getTransform();

        AffineTransform newAT = new AffineTransform(saveAT);

        final String labelY = "Time [" + ruler.getUnitName() + "]";

        Point yLabelP = ScreenTools.getStringPosition(g, labelY, new Dimension(
                yAxisL, ruler.getLeftOffset()));

        int lpy = ruler.getHeight() - ruler.getBottomOffset() - yLabelP.x;
        int lpx = yLabelP.y;
        LOGGER.fine("ruler : " + ruler.getHeight() + " , " + ruler.getWidth());
        LOGGER.fine("yLabelP " + yLabelP);

        newAT.translate(lpx, lpy);
        newAT.rotate(-Math.PI / 2);
        g.setTransform(newAT);
        g.drawString(labelY, 0, 0);
        g.setTransform(saveAT);

        SwingArrowObject vertArrow = new SwingArrowObject(ARRAY_TYPE.VERTICAL,
                ruler.getLeftOffset(), ruler.getTopOffset(), ruler
                        .getArrowLength(), true);
        vertArrow.draw(g);

        LOGGER.fine("first tic at "
                + ruler.getScreenPositionX(ruler.getFirstXTicValue()));
        LOGGER.fine("last tic at "
                + ruler.getScreenPositionX(ruler.getLastXTicValue()));

        // printAxisX(g, xAxisY);
        printAxisX2(g, xAxisY);

        final double yTicHeight = (double) (yAxisL - ruler.getArrowLength())
                / ruler.getNumberOfTicsY();
        for (int i = 0; i < ruler.getNumberOfTicsY(); i++) {
            int ticPosY = yAxisStartY - (int) ((i + 1) * yTicHeight);
            g.drawLine(yAxisX, ticPosY, yAxisX - ruler.getTicLength(), ticPosY);
        }

        final String lastTicLabelY = ruler.getLastTicLabelY();
        g.drawString(lastTicLabelY, 10, ruler.getHeight()
                - ruler.getBottomOffset() - yAxisL);

        final String firstTicLabelY = ruler.getFirstTicLabelY();
        g.drawString(firstTicLabelY, 10, ruler.getHeight()
                - ruler.getBottomOffset());

    }

    private void printAxisX2(final Graphics2D g, final int xstart) {

        final double ticWidth = ((double) ruler.getScreenPositionX(ruler
                .getLastXTicValue()) - (double) ruler.getScreenPositionX(ruler
                .getFirstXTicValue()))
                / (double) ruler.getNumberOfTicsX();

        // LOGGER.info("ticWidth " + ticWidth);

        final String lastLabel = ruler
                .getXLabelAt(ruler.getNumberOfTicsX() - 1);
        // LOGGER.info("last label " + lastLabel);

        final int lastLabelW = ScreenTools.getStringWidth(g, lastLabel) + 4;
        // LOGGER.info("last label width " + lastLabelW);

        final int stepW = Math.max(1, (int) Math.ceil((double) lastLabelW
                / ticWidth));

        // LOGGER.info("stepW: " + stepW);

        for (int p = 0; p < ruler.getNumberOfTicsX(); p++) {

            final String xLabel = ruler.getXLabelAt(p);
            int ticPos = (int) ((p + 1) * ticWidth)
                    + ruler.getScreenPositionX(ruler.getFirstXTicValue());

            // LOGGER.info("ticPos " + ticPos);

            if (ticPos < ruler.getWidth() - ruler.getRightOffset()) {
                g.drawLine(ticPos, xstart, ticPos, xstart
                        + ruler.getTicLength());
                final Stroke saveStroke = g.getStroke();
                g.setStroke(gridStroke);
                g.setColor(Color.LIGHT_GRAY);
                g.drawLine(ticPos, ruler.getHeight() - ruler.getBottomOffset(),
                        ticPos, ruler.getTopOffset());
                g.setStroke(saveStroke);
                g.setColor(Color.BLACK);
            }

            final int labelW = ScreenTools.getStringWidth(g, xLabel);
            final int stringPos = (int) (ticPos - (int) (ticWidth / 2) - labelW / 2);
            if (stringPos > ruler.getLeftOffset()
                    && stringPos < ruler.getWidth() - ruler.getRightOffset()
                    || p == 0) {
                if (p % stepW == 0) {
                    g.drawString(xLabel, stringPos, xstart
                            + g.getFont().getSize() + 2);
                }
            }
        }
    }

}
