/**
 * jedule - schedule drawing tool
 *
 * Copyright (C) 2007
 * Sascha Hunold <hunoldinho@users.sourceforge.net>
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jedule.ui.swing.components;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;

import javax.swing.SwingUtilities;

import net.sf.jedule.graphics.GraphicsConstants;
import net.sf.jedule.graphics.JSheetObject;
import net.sf.jedule.graphics.GraphicsConstants.ZOOM_DIRECTION;
import net.sf.jedule.graphics.composed.JeduleGraphic;

/**
 * @author sascha
 *
 */
public class JeduleGraphicSwingComponent extends JSheetSwingComponent implements
		ComponentListener {

    private static final long serialVersionUID = 1L;
    private JeduleGraphic jedGraphic;
	private Point zoomPoint1;
	private Point zoomPoint2;

	private Point movePoint1;
	private Point movePoint2;

	private Point crossPointDisplay;
	private Point2D crossPointRuler;

	private BufferedImage schedImage;
	private boolean dirty = true;

	/**
	 * @param jedGraphic
	 */
	public JeduleGraphicSwingComponent(JeduleGraphic jedGraphic) {
		super(jedGraphic);
		addComponentListener(this);
		this.jedGraphic = jedGraphic;

		setDoubleBuffered(true);
	}

	public void setJeduleGraphic(JeduleGraphic graphic) {
		this.jedGraphic = graphic;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see net.sf.jedule.ui.swing.IJSheetSwingComponent#renderComponent(java.awt.Graphics2D)
	 */
	public void renderComponent(Graphics2D g) {
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, jedGraphic.getWidth() - 1, jedGraphic.getHeight() - 1);
	}

	public JSheetObject getJSheetObject() {
		return this.jedGraphic;
	}

	protected void setDirty(boolean dirty) {
		this.dirty = dirty;
	}

	protected boolean isDirty() {
		return this.dirty;
	}



	/* (non-Javadoc)
	 * @see net.sf.jedule.ui.swing.components.JSheetSwingComponent#doLayout()
	 */
	@Override
	public void doLayout() {
		super.doLayout();
		setDirty(true);
	}

	public void paint(Graphics g) {

		boolean renderImage = false;
	    if (schedImage == null) {
	    	renderImage = true;
	    } else 	if( schedImage.getHeight() != getHeight() && schedImage.getWidth() != getWidth() ) {
	    	renderImage = true;
	    } else if( isDirty() ) {
	    	renderImage = true;
	    }

	    if( renderImage ) {
	    	schedImage = (BufferedImage)createImage(getWidth(), getHeight());
	    	Graphics imGraphics = schedImage.getGraphics();
			super.paint(imGraphics);
			setDirty(false);
	    }


		Graphics2D g2 = (Graphics2D) g;

		g2.drawImage(schedImage, 0, 0, this);

		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		if (crossPointRuler != null) {
			StringBuffer posString = new StringBuffer();
			//posString.append((int)crossPointRuler.getX());
			//posString.append(",");
			posString.append(crossPointRuler.getY());

			paintCrosshairAt(g2, posString.toString(), crossPointDisplay.x,
					crossPointDisplay.y);

		}

		if (zoomPoint1 != null && zoomPoint2 != null) {
			Rectangle r = createRectangle(zoomPoint1, zoomPoint2);
			g2.setColor(Color.WHITE);
			g2.drawRect(r.x + 1, r.y + 1, r.width, r.height);
			g2.setColor(Color.BLACK);
			g2.drawRect(r.x, r.y, r.width, r.height);
		}

		if (movePoint1 != null && movePoint2 != null) {

			double x0 = movePoint1.x;
			double y0 = movePoint1.y;

			double x1 = movePoint2.x;
			double y1 = movePoint2.y;

			// make the arrow start not directly above the mouse cursor
			double length = Math.sqrt((x1 - x0) * (x1 - x0) + (y1 - y0)
					* (y1 - y0));
			double newLength = length - 8;

			x1 = x0 + newLength * (x1 - x0) / length;
			y1 = y0 + newLength * (y1 - y0) / length;

			// (x2,y2) normal vector, orthogonal to line vector (x0-x1, y0-y1)
			double x2 = 1.0;

			double y2 = 0.0;
			if ((y0 - y1) != 0.0) {
				y2 = -(x0 - x1) / (y0 - y1);
			} else {
				x2 = 0.0;
				y2 = 1.0;
			}

			x2 = x2 / Math.sqrt(x2 * x2 + y2 * y2);
			y2 = y2 / Math.sqrt(x2 * x2 + y2 * y2);

			//System.out.println("x2,y2:" + x2 + "," + y2);

			// (x3,y3) point on line which is length of arrow head
			double x3 = x1
					+ 10
					* ((x0 - x1) / Math.sqrt((x0 - x1) * (x0 - x1) + (y0 - y1)
							* (y0 - y1)));
			double y3 = y1
					+ 10
					* ((y0 - y1) / Math.sqrt((x0 - x1) * (x0 - x1) + (y0 - y1)
							* (y0 - y1)));

			//System.out.println("x3,y3:" + x3 + "," + y3);

			// (x4,y4) and (x5,y5) are points of arrow ends
			double x4 = x3 + 5 * x2;
			double y4 = y3 + 5 * y2;

			double x5 = x3 - 5 * x2;
			double y5 = y3 - 5 * y2;

			g2.setColor(Color.BLACK);
			g2.drawLine((int) x0, (int) y0, (int) x1, (int) y1);
			//      g2.drawLine((int)x1, (int)y1, (int)x4, (int)y4);
			//      g2.drawLine((int)x1, (int)y1, (int)x5, (int)y5);

			g2.fillPolygon(
					new int[] { (int) x1, (int) x5, (int) x4, (int) x1 },
					new int[] { (int) y1, (int) y5, (int) y4, (int) y1 }, 4);

		}

	}

	/**
	 * @param g
	 * @param string
	 * @param x
	 * @param y
	 */
	private void paintCrosshairAt(Graphics2D g, String s, int x, int y) {
		g.drawLine(x, y - 7, x, y + 7);
		g.drawLine(x - 7, y, x + 7, y);
		g.drawString(s, x + 2, y - 2);
	}

	public Rectangle createRectangle(Point p1, Point p2) {
		final int pX = Math.min(p1.x, p2.x);
		final int pY = Math.min(p1.y, p2.y);
		final int pW = Math.abs(p2.x - p1.x);
		final int pH = Math.abs(p2.y - p1.y);
		return new Rectangle(pX, pY, pW, pH);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see net.sf.jedule.ui.swing.components.JSheetSwingComponent#customMousePressed(java.awt.event.MouseEvent)
	 */
	@Override
	public void customMousePressed(MouseEvent e) {
		if (SwingUtilities.isLeftMouseButton(e)) {
			zoomPoint1 = e.getPoint();
			setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
		}
		if (SwingUtilities.isRightMouseButton(e)) {
			movePoint1 = e.getPoint();
			setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
		}
		updateCrosshair(null);
		repaint();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see net.sf.jedule.ui.swing.components.JSheetSwingComponent#customMouseReleased(java.awt.event.MouseEvent)
	 */
	@Override
	public void customMouseReleased(MouseEvent e) {
		if (zoomPoint1 != null && zoomPoint2 != null) {
			final Rectangle zoomRect = createRectangle(zoomPoint1, zoomPoint2);
			jedGraphic.zoom(zoomRect);
			jedGraphic.doLayout();
			doLayout();
		}
		zoomPoint1 = null;
		zoomPoint2 = null;

		if (movePoint1 != null && movePoint2 != null) {
			//int xMove = movePoint1.x - movePoint2.x;
			//int yMove = movePoint2.y - movePoint2.y;
			jedGraphic.move(movePoint1, movePoint2);
			jedGraphic.doLayout();
			doLayout();
		}
		movePoint1 = null;
		movePoint2 = null;

		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		repaint();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see net.sf.jedule.ui.swing.components.JSheetSwingComponent#customMouseDragged(java.awt.event.MouseEvent)
	 */
	@Override
	public void customMouseDragged(MouseEvent e) {
		if (zoomPoint1 != null && SwingUtilities.isLeftMouseButton(e)) {
			zoomPoint2 = e.getPoint();
			repaint();
		}
		if (movePoint1 != null && SwingUtilities.isRightMouseButton(e)) {
			movePoint2 = e.getPoint();
			repaint();
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see net.sf.jedule.ui.swing.components.JSheetSwingComponent#customMouseWheelMoved(java.awt.event.MouseWheelEvent)
	 */
	@Override
	public void customMouseWheelMoved(MouseWheelEvent e) {
		final float scaleFac = 0.20f;
		float scaling = 1.0f + (float) e.getWheelRotation() * scaleFac;
		if (scaling < 0.0f) {
			scaling = 0.1f;
		}

		GraphicsConstants.ZOOM_DIRECTION direction = ZOOM_DIRECTION.BOTH_AXES;
		if (e.isAltDown()) {
			direction = ZOOM_DIRECTION.X_AXIS;
		} else if (e.isControlDown()) {
			direction = ZOOM_DIRECTION.Y_AXIS;
		}

		jedGraphic.zoom(scaling, e.getPoint(), direction);
		jedGraphic.doLayout();
		doLayout();

		updateCrosshair(e);

		repaint();
	}

	/* (non-Javadoc)
	 * @see net.sf.jedule.ui.swing.components.JSheetSwingComponent#customMouseMoved(java.awt.event.MouseEvent)
	 */
	@Override
	public void customMouseMoved(MouseEvent e) {

		if (e.getButton() == MouseEvent.NOBUTTON) {
			updateCrosshair(e);
			repaint();
		}

	}

	/**
	 *
	 */
	private void updateCrosshair(MouseEvent e) {
		if (e == null) {
			crossPointDisplay = null;
			crossPointRuler = null;
		} else {
			crossPointDisplay = e.getPoint();
			crossPointRuler = jedGraphic.getCoordinates(e.getPoint());
		}
	}

	/**
	 *
	 */
	public void resetView() {
		jedGraphic.resetView();
	}

	/**
	 *
	 */
	public void resize() {
		jedGraphic.setSize(getWidth(), getHeight());
		jedGraphic.doLayout();
		doLayout();
		repaint();
	}

	/**
	 *
	 */
	public void setBoundsY(double yMin, double yMax) {
		jedGraphic.setBoundsY(yMin, yMax);
		doLayout();
	}

	public double getMinY() {
		return jedGraphic.getMinY();
	}

	public double getMaxY() {
		return jedGraphic.getMaxY();
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentResized(java.awt.event.ComponentEvent)
	 */
	public void componentResized(ComponentEvent e) {
		resize();
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
	 */
	public void componentHidden(ComponentEvent e) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentMoved(java.awt.event.ComponentEvent)
	 */
	public void componentMoved(ComponentEvent e) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see java.awt.event.ComponentListener#componentShown(java.awt.event.ComponentEvent)
	 */
	public void componentShown(ComponentEvent e) {
		// TODO Auto-generated method stub

	}

}
